package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.dto.PrintTaskDto;
import com.qmth.distributed.print.business.bean.dto.PrintTaskTotalDto;
import com.qmth.distributed.print.business.bean.result.ExaminationDetailResult;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.bean.result.SummarizedDataResult;
import com.qmth.distributed.print.business.entity.ExamDetail;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 考务数据 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface ExamDetailService extends IService<ExamDetail> {
    /**
     * 计算印刷计划下的总科次
     * @param printPlanId 印刷计划id
     * @return 该印刷计划下总科次
     */
    double calculateTotalSubjects(Long printPlanId);

    /**
     * 计算印刷计划下的总卷袋数
     * @param printPlanId 印刷计划id
     * @return 该印刷计划下总卷袋数
     */
    double calculateTotalPackages(Long printPlanId);

    IPage<PrintTaskDto> listPrintTask(Long printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Integer pageNumber, Integer pageSize);

    PrintTaskTotalDto taskTotalData(Long printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime);

    String taskViewPDF(Long examDetailId);

    /**
     * 查询有效的考务字段
     * @param schoolId 学校id
     * @return 考务字段
     */
    List<FieldsDto> findExaminationFields(Long schoolId);

    /**
     * 考务数据模板下载
     * @param schoolId 学校id
     * @return 是否成功
     */
    void downLoadExaminationTemplate(Long schoolId, HttpServletResponse response) throws IOException;


    /**
     * 查询考务数据-简略
     * @param schoolId 学校id
     * @param printPlanId 印刷计划id
     * @param courseCode 课程代码
     * @param paperNumber 试卷编号
     * @param examPlace 考点
     * @param examRoom 考场
     * @param packageCode 卷袋编号
     * @param pageNumber 分页页码
     * @param pageSize 分页数量
     * @return 查询结果
     */
    IPage<ExaminationResult> findExaminationBriefPage(Long schoolId,Long printPlanId,String courseCode,String paperNumber,
                                                      String examPlace,String examRoom,String packageCode,int pageNumber,int pageSize);

    /**
     * 查询考务数据-汇总数据查询
     * @param schoolId 学校id
     * @param printPlanId 印刷计划id
     * @param courseCode 课程代码
     * @param paperNumber 试卷编号
     * @param examPlace 考点
     * @param examRoom 考场
     * @param packageCode 卷袋编号
     * @return 查询结果
     */
    SummarizedDataResult findSummarizedData(Long schoolId,Long printPlanId,String courseCode,String paperNumber,
                                            String examPlace,String examRoom,String packageCode);


    /**
     * 查询考务数据-明细
     * @param printPlanId 印刷计划id
     * @param courseCode 课程代码
     * @param paperNumber 试卷编号
     * @param examPlace 考点
     * @param examRoom 考场
     * @param studentParam 学生查询参数
     * @param pageNumber 分页页码
     * @param pageSize 分页数量
     * @return 查询结果
     */
    IPage<ExaminationDetailResult> findExaminationDetail(Long printPlanId,String courseCode,String paperNumber,String examPlace,
                                                         String examRoom,String studentParam,int pageNumber,int pageSize);

    IPage<ExaminationDetailResult> findExaminationDetail(Long convertIdToLong, int pageNumber, int pageSize);

    /**
     * 查询印刷计划下考点数据源
     * @return 考点集合
     */
    List<String> findExamPlaceDatasource();

    /**
     * 查询印刷计划下考场数据源
     * @return 考场集合
     */
    List<String> findExamRoomDatasource();

    boolean submitTask(ExamDetail examDetail);

    boolean taskCancel(ExamDetail examDetail);

    boolean resubmitTask(ExamDetail examDetail);

    void disposeExamDetailByExaminationExcel(List<Map<String, Object>> dataList, Long userId);

    void disposeExamDetailCourseByExaminationExcel(List<Map<String, Object>> dataList,Long userId);

    void disposeExamStudentByExaminationExcel(List<Map<String, Object>> dataList,Long userId);
}
