package com.qmth.distributed.print.business.templete.execute;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.SysRole;
import com.qmth.distributed.print.business.entity.TBTask;
import com.qmth.distributed.print.business.enums.RoleTypeEnum;
import com.qmth.distributed.print.business.enums.TaskResultEnum;
import com.qmth.distributed.print.business.enums.TaskStatusEnum;
import com.qmth.distributed.print.business.service.SysRoleService;
import com.qmth.distributed.print.business.templete.importData.AsyncImportTaskTemplete;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.*;

/**
 * @Description: 用户导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class AsyncUserImportTempleteService extends AsyncImportTaskTemplete {
    private final static Logger log = LoggerFactory.getLogger(AsyncUserImportTempleteService.class);

    static final String OBJ_TITLE = "用户数据";

    @Resource
    TaskLogicService taskLogicService;

    @Resource
    SysRoleService sysRoleService;

    /**
     * 异步处理导入
     *
     * @param map
     * @return
     */
    @Override
    public Result importTask(Map<String, Object> map) throws IOException {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        InputStream inputStream = super.getUploadFileInputStream(tbTask);
        StringJoiner stringJoinerSummary = new StringJoiner("\n")
                .add(MessageFormat.format("{0}{1}{2}", FORMAT_TIME, BEGIN_TITLE, OBJ_TITLE));
        tbTask.setStatus(TaskStatusEnum.RUNNING);
        try {
            List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(inputStream, Lists.newArrayList(UserImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
                List<ExcelError> excelErrorTemp = new ArrayList<>();
                for (int i = 0; i < finalExcelList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                    List<Object> userImportDtoList = excelMap.get(i);
                    for (int y = 0; y < userImportDtoList.size(); y++) {
                        UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                        excelErrorTemp.addAll(ExcelUtil.checkExcelField(userImportDto, y, i));
                        //查角色名是否存在
                        QueryWrapper<SysRole> sysRoleQueryWrapper = new QueryWrapper<>();
                        sysRoleQueryWrapper.lambda().eq(SysRole::getName, userImportDto.getRole());
                        SysRole sysRole = sysRoleService.getOne(sysRoleQueryWrapper);
                        if (Objects.isNull(sysRole)) {
                            throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[角色名]不存在")));
                        }
                        if (sysRole.getType() == RoleTypeEnum.SCHOOL_ADMIN && Objects.isNull(userImportDto.getSchoolCode())) {
                            excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]必填"));
                        }
                        if (userImportDto.getPassword().length() < 6) {
                            excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[密码]不能小于6位"));
                        } else if (userImportDto.getPassword().length() > 30) {
                            excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[密码]不能大于30位"));
                        }
                        if (userImportDto.getRealName().length() > 25) {
                            excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[姓名]不能超过25个字符"));
                        }
                        if (userImportDto.getLoginName().length() > 25) {
                            excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]不能超过25个字符"));
                        }
                        userImportDto.setSysRole(sysRole);
                    }
                }
                if (excelErrorTemp.size() > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(excelErrorTemp));
                }
                return finalExcelList;
            });
//            Map<String, Object> mapLogic = taskLogicService.executeUserImportLogic(finalList, map);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, FINISH_TITLE, 111, FINISH_SIZE));
            tbTask.setResult(TaskResultEnum.SUCCESS);
        } catch (Exception e) {
            log.error("请求出错", e);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, EXCEPTION_TITLE, EXCEPTION_DATA, e.getMessage()));
            tbTask.setResult(TaskResultEnum.ERROR);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {//生成txt文件
            tbTask.setSummary(stringJoinerSummary.toString());
            super.createTxt(tbTask);
        }
        return ResultUtil.ok(map);
    }
}
