package com.qmth.distributed.print.business.templete.service.impl;

import cn.hutool.core.io.IoUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.DrawRuleEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.*;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Base64Util;
import com.qmth.distributed.print.common.util.HexUtils;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.*;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    ExamDetailService examDetailService;

    @Resource
    OssUtil ossUtil;

    @Resource
    CacheService cacheService;

    @Resource
    FreemarkerUtil freemarkerUtil;

    /**
     * 执行导入用户逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Gson gson = new Gson();
        LinkedMultiValueMap<Long, SysUser> tbUserLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> userImportDtoList = excelMap.get(i);
            for (int y = 0; y < userImportDtoList.size(); y++) {
                UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                SysUser sysUser = gson.fromJson(gson.toJson(userImportDto), SysUser.class);

                //查用户登录名是否重复
                QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
                sysUserQueryWrapper.lambda().eq(SysUser::getLoginName, userImportDto.getLoginName());
                long count = sysUserService.count(sysUserQueryWrapper);
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]重复")));
                }
                //查学校是否存在
                if (Objects.nonNull(userImportDto.getSchoolCode())) {
                    QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
                    basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, userImportDto.getSchoolCode());
                    BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
                    if (Objects.isNull(basicSchool)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]不存在")));
                    }
                    sysUser.setSchoolId(basicSchool.getId());
                }
                sysUser.setInsertInfo(requestUser.getId());
                sysUser.setPassword(Base64Util.encode(userImportDto.getPassword().getBytes()));
                tbUserLinkedMultiValueMap.add(userImportDto.getSysRole().getId(), sysUser);
            }
        }
        tbUserLinkedMultiValueMap.forEach((k, v) -> {
            sysUserService.saveBatch(v);
            v.forEach(s -> {
                commonService.addUserRolePrivilege(s, k);
            });
        });
        return map;
    }

    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) throws IOException {
        List<BasicAttachment> basicAttachmentList = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            Long examDetailId = Objects.nonNull(map.get("examDetailId")) ? (Long) map.get("examDetailId") : null;

            //这里为保存附件html的逻辑
            //查询题卡详情是否有附件id，有的话则把以前的附件删除
            Set<Long> attachmentIds = null;
            Set<File> ftlList = null;

            //查询printPlan
            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
            if (Objects.isNull(examPrintPlan)) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
            }

            BasicSchool basicSchool = cacheService.schoolCache(examPrintPlan.getSchoolId());

            //查询examDetail
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
            if (Objects.nonNull(examDetailId)) {
                examDetailQueryWrapper.lambda().eq(ExamDetail::getId, examDetailId);
            }
            List<ExamDetail> examDetailList = detailService.list(examDetailQueryWrapper);
            if (Objects.isNull(examDetailList) || examDetailList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("考务计划为空");
            }
            attachmentIds = Objects.isNull(attachmentIds) ? attachmentIds = new HashSet<>() : attachmentIds;
            ftlList = Objects.isNull(ftlList) ? ftlList = new HashSet<>() : ftlList;
            for (ExamDetail examDetail : examDetailList) {
                //查询examDetailCourse
                QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
                examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
                List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);

                List<String> variablePdfList = new ArrayList<>();//变量印品(签到表、卷袋贴)
                List<String> ordinaryPdfList = new ArrayList<>();//普通印品(登记表)
                List<String> paperPdfList = new ArrayList<>();//所有试卷
                List<String> examStudentPdfList = new ArrayList<>();//所有题卡
                List<String> backupPaperPdfList = new ArrayList<>();//备份试卷
                List<String> cardPdfList = new ArrayList<>();//备份题卡

                if (Objects.nonNull(examPrintPlan.getOrdinaryContent())) {
                    //获取普通印品
                    JSONArray jsonArrayOrdinary = JSONArray.parseArray(examPrintPlan.getOrdinaryContent());
                    for (int i = 0; i < jsonArrayOrdinary.size(); i++) {
                        JSONObject jsonObjectOrdinary = jsonArrayOrdinary.getJSONObject(i);
                        Long attachmentId = (Long) jsonObjectOrdinary.get("attachmentId");
                        BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                        createCheckIn(basicAttachment, ordinaryPdfList, (Integer) jsonObjectOrdinary.get("backupCount"));
                    }
                }

                for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                    List<String> paperPdfTempList = new ArrayList<>();//所有试卷
                    List<String> examStudentTempPdfList = new ArrayList<>();//所有题卡
                    List<String> backupPaperTempPdfList = new ArrayList<>();//备份试卷
                    List<String> cardPdfTempList = new ArrayList<>();//备份题卡

                    //查询试卷
                    QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
                    examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                            .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                            .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                            .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                            .eq(ExamTask::getEnable, true)
                            .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
                    List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
                    if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                        throw ExceptionResultEnum.ERROR.exception("命题任务为空");
                    }
                    Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

                    QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
                    examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                            .eq(ExamTaskDetail::getEnable, true);
                    List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
                    JSONObject jsonObject = new JSONObject();
                    JSONArray jsonArray = new JSONArray();
                    for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                        //查询题卡
                        ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
                        if (Objects.isNull(examCard)) {
                            throw ExceptionResultEnum.ERROR.exception("题卡为空");
                        }
                        QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
                        examCardDetailQueryWrapper.lambda().eq(ExamCardDetail::getCardId, examCard.getId());
                        List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

                        //抽取卷型
                        String paperType = getPaperType(examPrintPlan, examTaskDetail);

                        //获取试卷pdf
                        getPaperPdf(examTaskDetail, examPrintPlan.getBackupCount(), paperPdfTempList, backupPaperTempPdfList);
                        paperPdfList.addAll(paperPdfTempList);
                        backupPaperPdfList.addAll(backupPaperTempPdfList);

                        basicAttachmentList = Objects.isNull(basicAttachmentList) ? basicAttachmentList = new ArrayList<>() : basicAttachmentList;
                        for (ExamCardDetail examCardDetail : examCardDetailList) {
                            getCardAttachmentId(examCardDetail, attachmentIds);

                            //把模板页面上的 ${} 替换成实际内容
                            String cardContent = replaceHtmlTemplete(examCardDetail);
                            String studentContent = cardContent;

                            for (int i = 1; i <= examPrintPlan.getBackupCount(); i++) {
                                basicAttachmentList.add(cardHtml(cardContent, paperType, examDetailCourse, examCard, jsonArray, sysUser.getId(), cardPdfTempList));
                            }
                            cardPdfList.addAll(cardPdfTempList);

                            //查询考生
                            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
                            examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                                    .eq(ExamStudent::getExamDetailCourseId, examTaskDetail.getId());
                            List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);

                            if (Objects.nonNull(examPrintPlan.getVariableContent())) {
                                //获取变量印品
                                JSONArray jsonArrayVariable = JSONArray.parseArray(examPrintPlan.getVariableContent());
                                for (int i = 0; i < jsonArrayVariable.size(); i++) {
                                    JSONObject jsonObjectVariable = jsonArrayVariable.getJSONObject(i);
                                    String type = (String) jsonObjectVariable.get("type");
                                    Long attachmentId = (Long) jsonObjectVariable.get("attachmentId");
                                    BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                                    ftlList.add(commonService.getFile(basicAttachment.getPath(), false));
                                    if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "SIGN")) {//签到表
                                        createSignBook(basicAttachment, basicSchool.getName(), examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                    } else if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "PACKAG")) {//卷袋贴
                                        createPaperPackage(basicAttachment, basicSchool.getName(), examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                    }
                                }
                            }

                            for (ExamStudent t : examStudentList) {
                                basicAttachmentList.add(examStudentHtml(attachmentIds, studentContent, t, paperType, examCard, sysUser.getId(), examStudentTempPdfList));
                            }
                            examStudentPdfList.addAll(examStudentTempPdfList);
                            examStudentService.saveOrUpdateBatch(examStudentList);
                            jsonObject.put("card", jsonArray);
                            examCardDetail.setAttachmentId(jsonObject.toJSONString());
                        }
                        examCardDetailService.saveOrUpdateBatch(examCardDetailList);
                    }
                    examTaskDetailService.saveOrUpdateBatch(examTaskDetailList);

                    Integer paperPageA3 = 0;
                    Integer cardPageA3 = 0;
                    paperPageA3 = paperPageA3 + paperPdfTempList.size() + backupPaperTempPdfList.size();
                    cardPageA3 = cardPageA3 + examStudentTempPdfList.size() + cardPdfTempList.size();
                    examDetailCourse.setPaperPagesA3(paperPageA3);
                    examDetailCourse.setCardPagesA3(cardPageA3);
                }
                detailCourseService.saveOrUpdateBatch(examDetailCourseList);
                //合并pdf
                basicAttachmentList.add(mergePdf(tbTask, examDetail, sysUser.getId(), variablePdfList, ordinaryPdfList, paperPdfList, examStudentPdfList, backupPaperPdfList, cardPdfList));
            }
            map.computeIfAbsent("size", v -> examDetailList.size());
            //最后一步删除附件
            deleteAttachment(attachmentIds, ftlList);
        } catch (Exception e) {
            log.error("请求出错", e);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeExaminationLogic(Map<String, Object> map) throws Exception {
//        printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode,
        Long printPlanId = null;
        if (map.get("printPlanId") != null) {
            printPlanId = SystemConstant.convertIdToLong(String.valueOf(map.get("printPlanId")));
        }
        String courseCode = null;
        if (map.get("courseCode") != null) {
            courseCode = String.valueOf(map.get("courseCode"));
        }
        String paperNumber = null;
        if (map.get("paperNumber") != null) {
            paperNumber = String.valueOf(map.get("paperNumber"));
        }
        String examPlace = null;
        if (map.get("examPlace") != null) {
            examPlace = String.valueOf(map.get("examPlace"));
        }
        String examRoom = null;
        if (map.get("examRoom") != null) {
            examRoom = String.valueOf(map.get("examRoom"));
        }
        String packageCode = null;
        if (map.get("packageCode") != null) {
            packageCode = String.valueOf(map.get("packageCode"));
        }

        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        Long userId = tbTask.getCreateId();
        Long schoolId = tbTask.getSchoolId();

        List<ExaminationResult> examinationResultList = examDetailService.findExaminationBriefPage(schoolId,
                printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode,
                SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE).getRecords();
        System.out.println("examinationResultList = " + JSON.toJSONString(examinationResultList));

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ExcelUtil.excelMake(ExaminationResult.class, examinationResultList, outputStream);
        InputStream in = new ByteArrayInputStream(outputStream.toByteArray());
        IoUtil.copy(in, outputStream);

//        UploadFileEnum.FILE/
        LocalDateTime nowTime = LocalDateTime.now();
        StringJoiner stringJoiner = new StringJoiner("");
        stringJoiner.add(UploadFileEnum.FILE.getTitle()).add(File.separator);

        stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                .add(String.format("%02d", nowTime.getDayOfMonth()));

        JSONObject jsonObject = new JSONObject();
        stringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(".").add(SystemConstant.XLSX);

        String dirName = stringJoiner.toString().replaceAll("\\\\", "/");
//        File file = new File(dirName);
//        ConvertUtil.inputStream2File(in,file);
        ossUtil.ossUpload(dirName, in, DigestUtils.md5Hex(in));
        jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
        jsonObject.put(SystemConstant.PATH, dirName);
        jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE.getTitle());
        map.put("path", jsonObject.toString());
        return map;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeImportExaminationLogic(Map<String, Object> map) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long printPlanId = tbTask.getPrintPlanId();
        Long userId = tbTask.getCreateId();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);

        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划不存在printPlanId ： " + printPlanId);
        }
        String printPlanName = examPrintPlan.getName();
        Long schoolId = tbTask.getSchoolId();
        // 该学校有效考务数据
        List<FieldsDto> fieldsDtoList = examDetailService.findExaminationFields(schoolId);
        if (fieldsDtoList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("该学校没有设置考务数据");
        }
        String importFilePath = tbTask.getImportFilePath();
        Map importFilePathMap = JSONObject.parseObject(importFilePath);
        String path = String.valueOf(importFilePathMap.get("path"));
        System.out.println("path = " + path);
        Workbook workbook = null;
        if (path.endsWith(SystemConstant.XLSX)) {
            workbook = new XSSFWorkbook(inputStream);
        } else if (path.endsWith("xls")) {
            workbook = new HSSFWorkbook(inputStream);
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件格式异常");
        }
        // 读取第一个工作表
        Sheet sheet = workbook.getSheetAt(0);
        // 获取sheet行数
        int totalRows = sheet.getPhysicalNumberOfRows();
        // 获取sheet列数
        int totalCells = 0;
        if (totalRows > 1 && sheet.getRow(0) != null) {
            totalCells = sheet.getRow(0).getPhysicalNumberOfCells();
        }
        System.out.println("行 ： " + totalRows + "\n列 ： " + totalCells);
        Row head = sheet.getRow(0);
        List<String> headList = new ArrayList<>();
        // 将必填字段匹配excel解析的表头索引
        for (int i = 0; i < totalCells; i++) {
            String cellValue = head.getCell(i).getStringCellValue();
            for (FieldsDto fieldsDto : fieldsDtoList) {
                if (cellValue.equals(fieldsDto.getName())) {
                    fieldsDto.setIndex(i);
                }
            }
            headList.add(cellValue);
        }

        // 搜索所有有效字段 excel中的表头是否包含
        for (FieldsDto fieldsDto : fieldsDtoList) {
            if (!headList.contains(fieldsDto.getName())) {
                System.out.println("exception " + fieldsDto.getName());
                throw ExceptionResultEnum.ERROR.exception("学校考务必填字段 ：'" + fieldsDto.getName() + "' 不存在");
            }
        }

        System.out.println("headList : " + headList);
        System.out.println("fieldsDtoList : " + JSONObject.toJSONString(fieldsDtoList));

        List<Map<String, Object>> dataList = new ArrayList<>();

        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);

            String studentCode = null;
            String studentName = null;
            String courseCode = null;
            String courseName = null;
            String examPlace = null;
            String examRoom = null;
            String examDate = null;
            String examTime = null;
            String paperNumber = null;
            List<FieldsDto> secondaryFieldList = new ArrayList<>();
            for (FieldsDto fieldsDto : fieldsDtoList) {
                String name = fieldsDto.getName();
                String code = fieldsDto.getCode();
                int index = fieldsDto.getIndex();
                String level = fieldsDto.getLevel();
                if ("学号".equals(name)) {
                    studentCode = row.getCell(index).getStringCellValue();
                } else if ("姓名".equals(name)) {
                    studentName = row.getCell(index).getStringCellValue();
                } else if ("课程代码".equals(name)) {
                    courseCode = row.getCell(index).getStringCellValue();
                } else if ("课程名称".equals(name)) {
                    courseName = row.getCell(index).getStringCellValue();
                } else if ("考点".equals(name)) {
                    examPlace = row.getCell(index).getStringCellValue();
                } else if ("考场".equals(name)) {
                    examRoom = row.getCell(index).getStringCellValue();
                } else if ("考试日期".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examDate = row.getCell(index).getStringCellValue();
                } else if ("考试时间".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examTime = row.getCell(index).getStringCellValue();
                } else if ("试卷编号".equals(name)) {
                    paperNumber = row.getCell(index).getStringCellValue();
                } else {
                    if ("primary".equals(level)) {
                        throw ExceptionResultEnum.ERROR.exception("有数据库不需要的必选字段 ： " + name);
                    }
                    if ("secondary".equals(level)) {
                        // 备选字段处理
                        FieldsDto secondaryField = new FieldsDto();
                        secondaryField.setLevel(level);
                        secondaryField.setCode(code);
                        secondaryField.setName(name);
                        secondaryField.setEnable(true);
                        secondaryField.setValue(row.getCell(index).getStringCellValue());
                        secondaryFieldList.add(secondaryField);
                    }
                }
            }
            Map<String, Object> dataMap = new HashMap<>();
            dataMap.put("studentCode", studentCode);
            dataMap.put("studentName", studentName);
            // TODO: 2021/4/9 校验课程
            dataMap.put("courseCode", courseCode);
            dataMap.put("courseName", courseName);
            dataMap.put("examPlace", examPlace);
            dataMap.put("examRoom", examRoom);
            // TODO: 2021/4/9 解析时间
            dataMap.put("examDate", examDate);
            dataMap.put("examTime", examTime);
            dataMap.put("paperNumber", paperNumber);
            dataMap.put("secondaryFieldList", secondaryFieldList);
            dataMap.put("schoolId", schoolId);
            dataMap.put("printPlanId", printPlanId);
            dataMap.put("printPlanName", printPlanName);
            dataList.add(dataMap);
        }
        System.out.println("dataList = " + JSON.toJSONString(dataList));


        // 组装exam_detail数据
        examDetailService.disposeExamDetailByExaminationExcel(dataList, userId);
        // 组装exam_detail_course数据
        examDetailService.disposeExamDetailCourseByExaminationExcel(dataList, userId);
        // 组装exam_student数据
        examDetailService.disposeExamStudentByExaminationExcel(dataList, userId);
        return null;
    }

    /**
     * 创建登记表
     *
     * @param basicAttachment
     * @param ordinaryPdfList
     * @param printCount
     */
    private void createCheckIn(BasicAttachment basicAttachment, List<String> ordinaryPdfList, Integer printCount) throws IOException {
        String type = basicAttachment.getType();
        JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
        if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.PDF_PREFIX)) {
            String filePath = (String) jsonObject.get(SystemConstant.PATH);
            String url = SystemConstant.TEMP_FILES_DIR + File.separator + filePath;
            File localFile = ossUtil.ossDownload(filePath, url);
            for (int i = 0; i < printCount; i++) {
                ordinaryPdfList.add(localFile.getPath());
            }
        } else {
            basicAttachmentService.saveAttachmentPdf(basicAttachment, ordinaryPdfList, printCount);
        }
    }

    /**
     * 生成卷袋贴
     *
     * @param basicAttachment
     * @param schoolName
     * @param examDetailCourse
     * @param examStudentList
     * @param variablePdfList
     * @param printCount
     * @throws IOException
     */
    private void createPaperPackage(BasicAttachment basicAttachment, String schoolName, ExamDetailCourse examDetailCourse, List<ExamStudent> examStudentList, List<String> variablePdfList, Integer printCount) throws IOException {
        Map<String, Object> htmlMap = new HashMap<>();
        htmlMap.put("examDetailCourseId", examDetailCourse.getId());
        htmlMap.put("schoolName", schoolName);
        htmlMap.put("examName", "测试exam1");
        htmlMap.put("courseName", examDetailCourse.getCourseName());
        htmlMap.put("courseCode", examDetailCourse.getCourseCode());
        htmlMap.put("examSite", "测试examSite1");
        htmlMap.put("examRoom", "测试examRoom1");
        htmlMap.put("startTime", "2021-04-13 00:00:00");
        htmlMap.put("endTime", "2021-04-14 00:00:00");
        htmlMap.put("paperCode", examDetailCourse.getPaperNumber());
        htmlMap.put("paperCodeImg", GoogleBarCodeUtil.createBarCode(examDetailCourse.getPaperNumber(), false));

        htmlMap.put("startCollege", "测试startCollege1");
        htmlMap.put("className", "测试className1");
        htmlMap.put("proctorCollege", "测试proctorCollege1");
        htmlMap.put("examManager", "测试examManager1");

        htmlMap.put("minSite", "测试minSite1");
        htmlMap.put("maxSite", "测试maxSite1");
        htmlMap.put("studentCount", examStudentList.size());

        htmlMap.put("tag", "测试tag1");

        htmlMap.computeIfAbsent("basicAttachment", v -> basicAttachment);
        htmlMap.computeIfAbsent("variablePdfList", v -> variablePdfList);
        htmlMap.computeIfAbsent("printCount", v -> printCount);
        freemarkerUtil.createPaperPackage(htmlMap);
    }

    /**
     * 创建签到表
     *
     * @param basicAttachment
     * @param schoolName
     * @param examDetailCourse
     * @param examStudentList
     * @param variablePdfList
     * @param printCount
     */
    private void createSignBook(BasicAttachment basicAttachment, String schoolName, ExamDetailCourse examDetailCourse, List<ExamStudent> examStudentList, List<String> variablePdfList, Integer printCount) throws IOException {
        Map<String, Object> htmlMap = new HashMap<>();
        htmlMap.put("examDetailCourseId", examDetailCourse.getId());
        htmlMap.put("schoolName", schoolName);
        htmlMap.put("courseName", examDetailCourse.getCourseName());
        htmlMap.put("courseCode", examDetailCourse.getCourseCode());
        htmlMap.put("examName", "测试exam1");
        htmlMap.put("examRoom", "测试examRoom1");
        htmlMap.put("classTeacher", "测试老师1");
        htmlMap.put("startTime", "2021-04-13 00:00:00");
        htmlMap.put("endTime", "2021-04-14 00:00:00");
        htmlMap.put("paperCode", examDetailCourse.getPaperNumber());
        htmlMap.put("paperCodeImg", GoogleBarCodeUtil.createBarCode(examDetailCourse.getPaperNumber(), false));

        int totalCount = examStudentList.size();
        if (totalCount > 0) {
            List<Map> subList = new ArrayList<>();
            int pageCount = totalCount % 80 == 0 ? totalCount / 80 : totalCount / 80 + 1;
            for (int i = 0; i < pageCount; i++) {
                Map subMap = new HashMap();
                subMap.put("index", i + 1);
                int studentCount;
                List<ExamStudent> subStudents;
                if (pageCount == 1) {
                    studentCount = totalCount;
                    subStudents = examStudentList;
                } else if (pageCount > 1 && i == pageCount - 1) {
                    studentCount = totalCount - (pageCount - 1) * 80;
                    subStudents = examStudentList.subList(80 * i, examStudentList.size());
                } else {
                    studentCount = 80;
                    subStudents = examStudentList.subList(80 * i, 80 * (i + 1));
                }
                subMap.put("studentCount", studentCount);
                List<ExamStudentDto> examStudentList1 = new ArrayList<>();
                List<ExamStudentDto> examStudentList2 = new ArrayList<>();

                Gson gson = new Gson();
                int mod = subStudents.size() % 2;
                int htmlTableCount = mod == 0 ? subStudents.size() / 2 : subStudents.size() / 2 + 1;
                for (int j = 0; j < htmlTableCount; j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentList1.add(examStudentDto);
                }
                for (int j = htmlTableCount; j < subStudents.size(); j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentList2.add(examStudentDto);
                }
                if (examStudentList1.size() > examStudentList2.size()) {
                    for (int j = subStudents.size() - htmlTableCount; j < examStudentList1.size(); j++) {
                        examStudentList2.add(new ExamStudentDto());
                    }
                }
                subMap.put("studentList1", examStudentList1);
                subMap.put("studentList2", examStudentList2);
                subList.add(subMap);
            }
            htmlMap.put("subList", subList);
        }
        htmlMap.computeIfAbsent("basicAttachment", v -> basicAttachment);
        htmlMap.computeIfAbsent("variablePdfList", v -> variablePdfList);
        htmlMap.computeIfAbsent("printCount", v -> printCount);
        freemarkerUtil.createSignBook(htmlMap);
    }


    /**
     * 替换html通用模版
     *
     * @param examCardDetail
     * @return
     */
    public String replaceHtmlTemplete(ExamCardDetail examCardDetail) {
        String content = examCardDetail.getHtmlContent();
        log.info("contentTemp:{}", content);
        content = content.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
        content = content.replaceAll("\\$\\{examNumberStr\\}", "");
        return content;
    }

    /**
     * 获取题卡attachmentId
     *
     * @param examCardDetail
     * @param attachmentIds
     */
    public void getCardAttachmentId(ExamCardDetail examCardDetail, Set<Long> attachmentIds) {
        if (Objects.nonNull(examCardDetail.getAttachmentId())) {
            JSONObject jsonObjectCard = JSONObject.parseObject(examCardDetail.getAttachmentId());
            JSONArray jsonArrayCard = (JSONArray) jsonObjectCard.get("card");
            for (int i = 0; i < jsonArrayCard.size(); i++) {
                JSONObject object = (JSONObject) jsonArrayCard.get(i);
                if (Objects.nonNull(object.get("attachmentId"))) {
                    attachmentIds.add(Long.parseLong(String.valueOf(object.get("attachmentId"))));
                }
            }
        }
    }

    /**
     * 获取试卷pdf
     *
     * @param examTaskDetail
     * @param backupCount
     * @param pdfList
     * @return
     * @throws IOException
     */
    public void getPaperPdf(ExamTaskDetail examTaskDetail, Integer backupCount, List<String>... pdfList) throws IOException {
        JSONObject jsonObjectPaper = JSONObject.parseObject(examTaskDetail.getPaperAttachmentIds());
        JSONArray jsonArrayPaper = jsonObjectPaper.getJSONArray("paper");
        for (int i = 0; i < jsonArrayPaper.size(); i++) {
            JSONObject object = (JSONObject) jsonArrayPaper.get(i);
            if (Objects.nonNull(object.get("attachmentId"))) {
                Long attachmentId = Long.parseLong(String.valueOf(object.get("attachmentId")));
                BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                String filePath = commonService.getFile(basicAttachment.getPath(), false).getPath();
                pdfList[0].add(filePath);
                for (int j = 1; j <= backupCount; j++) {
                    pdfList[1].add(filePath);
                }
            }
        }
    }

    /**
     * 抽取试卷
     *
     * @param examPrintPlan
     * @param examTaskDetail
     * @return
     */
    public String getPaperType(ExamPrintPlan examPrintPlan, ExamTaskDetail examTaskDetail) {
        //抽取卷型
        DrawRuleEnum drawRule = Objects.nonNull(examPrintPlan.getDrawRule()) ? examPrintPlan.getDrawRule() : DrawRuleEnum.ONE;
        //未曝光卷型
        String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
        //已曝光卷型
        String exposedPaperType = examTaskDetail.getExposedPaperType();
        String[] paperTypes = null;
        if (drawRule == DrawRuleEnum.ONE) {
            if (Objects.isNull(unexposedPaperType)) {
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            } else {
                paperTypes = unexposedPaperType.split("/");
            }
        } else {
            if (Objects.isNull(exposedPaperType) && Objects.isNull(unexposedPaperType)) {
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            }
            if (Objects.nonNull(unexposedPaperType)) {
                paperTypes = unexposedPaperType.split("/");
            } else {
                paperTypes = exposedPaperType.split("/");
            }
        }
        int paperRandom = new Random().nextInt(paperTypes.length);
        String paperType = paperTypes[paperRandom];
        examTaskDetail.setPaperType(paperType);
        return paperType;
    }

    /**
     * 合并pdf
     *
     * @param tbTask
     * @param examDetail
     * @param userId
     * @param list
     * @return
     * @throws IOException
     */
    public BasicAttachment mergePdf(TBTask tbTask, ExamDetail examDetail, Long userId, List<String>... list) throws IOException {
        StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
        List<String> mergePdfList = new ArrayList<>();
        int pageA3Count = 0;
        int pageA4Count = 0;
        for (int i = 0; i < list.length; i++) {
            if (i < 2) {
                pageA4Count = pageA4Count + list[i].size();
            } else {
                pageA3Count = pageA3Count + list[i].size();
            }
            mergePdfList.addAll(list[i]);
        }
        String dirName = PdfUtil.mergePdf(mergePdfList.toArray(new String[mergePdfList.size()]), null);
        File localPdfFile = new File(stringJoiner.toString() + File.separator + dirName);
        BasicAttachment basicAttachment = basicAttachmentService.saveAttachmentPdf(dirName, userId);
        tbTask.setImportFileName(basicAttachment.getName());
        tbTask.setImportFilePath(basicAttachment.getPath());
        examDetail.setAttachmentId(basicAttachment.getId());
        examDetail.setPagesA3(pageA3Count);
        examDetail.setPagesA4(pageA4Count);
        detailService.saveOrUpdate(examDetail);
        ossUtil.ossUpload(dirName, localPdfFile, BinaryUtil.toBase64String(HexUtils.decodeHex(DigestUtils.md5Hex(new FileInputStream(localPdfFile)))));
        localPdfFile.delete();
        for (String s : mergePdfList) {
            new File(s).delete();
        }
        return basicAttachment;
    }

    /**
     * 删除附件
     *
     * @param attachmentIds
     * @param ftlList
     */
    public void deleteAttachment(Set<Long> attachmentIds, Set<File> ftlList) {
        if (Objects.nonNull(attachmentIds) && attachmentIds.size() > 0) {
            QueryWrapper<BasicAttachment> basicAttachmentQueryWrapper = new QueryWrapper<>();
            basicAttachmentQueryWrapper.lambda().in(BasicAttachment::getId, attachmentIds);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentService.list(basicAttachmentQueryWrapper));
            basicAttachmentService.removeByIds(attachmentIds);
        }
        if (Objects.nonNull(ftlList) && ftlList.size() > 0) {
            for (File file : ftlList) {
                file.delete();
            }
        }
    }

    /**
     * 考生题卡html
     *
     * @param attachmentIds
     * @param studentContent
     * @param t
     * @param paperType
     * @param examCard
     * @param userId
     * @param examStudentPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment examStudentHtml(Set<Long> attachmentIds, String studentContent, ExamStudent t, String paperType, ExamCard examCard, Long userId, List<String> examStudentPdfList) throws IOException {
        if (Objects.nonNull(t.getAttachmentId())) {
            attachmentIds.add(t.getAttachmentId());
        }
        String studentHtml = studentContent;
        if (Objects.nonNull(t.getExtendFields())) {
            JSONArray jsonObjectExtend = (JSONArray) JSONArray.parse(t.getExtendFields());//扩展字段
            if (Objects.nonNull(jsonObjectExtend)) {
                for (int i = 0; i < jsonObjectExtend.size(); i++) {
                    JSONObject object = (JSONObject) jsonObjectExtend.get(i);
                    studentHtml = studentHtml.replaceAll("\\$\\{" + object.get("code") + "\\}", String.valueOf(object.get("value")));
                }
            }
        }
        //生成学生考号条码并将图片转成base64
        studentHtml = studentHtml.replaceAll("\\$\\{examNumber\\}", GoogleBarCodeUtil.createBarCode(t.getTicketNumber(), false));
        studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", t.getTicketNumber());
        //随机生成学生试卷条码并将图片转成base64
        studentHtml = studentHtml.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
        studentHtml = studentHtml.replaceAll("\\$\\{paperTypeName\\}", paperType);
        studentHtml = studentHtml.replaceAll("\\$\\{studentCode\\}", t.getStudentCode());
        studentHtml = studentHtml.replaceAll("\\$\\{name\\}", t.getStudentName());
        studentHtml = studentHtml.replaceAll("\\$\\{courseName\\}", examCard.getCourseName());

        //学生题卡
        BasicAttachment examStudentAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode() + "|" + t.getTicketNumber(), studentHtml, userId, examStudentPdfList);
        t.setAttachmentId(examStudentAttachment.getId());
        return examStudentAttachment;
    }

    /**
     * 通用题卡html
     *
     * @param cardContent
     * @param paperType
     * @param examDetailCourse
     * @param examCard
     * @param jsonArray
     * @param userId
     * @param cardPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment cardHtml(String cardContent, String paperType, ExamDetailCourse examDetailCourse, ExamCard examCard, JSONArray jsonArray, Long userId, List<String> cardPdfList) throws IOException {
        //通用题卡
        String cardTemp = cardContent;
        cardTemp = cardTemp.replaceAll("\\$\\{paperTypeName\\}", paperType);
        //随机生成试卷条码并将图片转成base64
        cardTemp = cardTemp.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
        //通用题卡生成卷袋贴条码
        String paperCode = examDetailCourse.getPaperNumber();
        String paperCodeImg = GoogleBarCodeUtil.createBarCode(paperCode, false);
        String paperCodeDiv = "<div class=\"page-box page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 80px;right: 35px;transform: rotate(-90deg);transform-origin: center right;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + paperCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + paperCode + "</p></div>";
        cardTemp = cardTemp.replaceAll("<div class=\"page-box page-box-0\">", paperCodeDiv);
        BasicAttachment cardAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode(), cardTemp, userId, cardPdfList);
        JSONObject object = new JSONObject();
        object.put("name", paperType);
        object.put("examDetailCourseId", examDetailCourse.getId());
        object.put("attachmentId", cardAttachment.getId());
        jsonArray.add(object);
        return cardAttachment;
    }
}
